import net.mikekohn.java_grinder.CPU;
import net.mikekohn.java_grinder.IOPort0;
import net.mikekohn.java_grinder.Timer;
import net.mikekohn.java_grinder.TimerListener;
import net.mikekohn.java_grinder.Watchdog;

// 16,000,000 cycles * 0.000050 = 800 cycles
// 800 cycles = 50us
// 20 interrupts = 1ms
// 40 interrupts = 2ms
// 400 interrupts = 20ms
// 20,000 interrupts = 1s

public class ChickenBatter implements TimerListener
{
  public static int debug = -1;
  public static int interrupt_count = 0;
  public static int servo_count = 0;
  public static int servo0_curr_value = 20;
  public static int servo1_curr_value = 20;
  public static int servo0_new_value = 20;
  public static int servo1_new_value = 20;

  public static void waitOneSecond()
  {
    // Spinning for 20,000 interrupts is 1 second.
    interrupt_count = 0;
    while(interrupt_count != 20000)
    {
      Watchdog.kick();
    }
  }

  public static void wait200ms()
  {
    // Spinning for 4,000 interrupts is 200 milliseconds.
    interrupt_count = 0;
    while(interrupt_count != 4000);
  }

  public static void main(String args)
  {
    CPU.setClock16();

    Watchdog.enable();

    IOPort0.setPinsAsOutput(0x26);
    IOPort0.setPinsLow(0xff);

    // Setup timer interrupt (aka, Java listener).
    Timer.setInterval(800, 1);
    Timer.setListener(true);

    while(true)
    {
      Watchdog.kick();

      if ((IOPort0.getPortInputValue() & 0x1) != 1)
      {
        IOPort0.setPinsHigh(0x20);
        int data = decodeRemote();
        IOPort0.setPinsLow(0x20);

        //if (data < 0 || data > 10) { debug = data; }
        //if (data == 0x7887)
        if (data == 0x20df)
        {
          waitOneSecond();
          waitOneSecond();
          waitOneSecond();

          servo0_new_value = 40;
          servo1_new_value = 40;

          waitOneSecond();

          servo0_new_value = 20;
          servo1_new_value = 20;
        }
      }
    }
  }

  public static int decodeRemote()
  {
    int data;
    int bit_count;

    // Wait header ON (180 interrupts = 9ms)
    interrupt_count = 0;

    while ((IOPort0.getPortInputValue() & 0x1) == 0)
    {
      Watchdog.kick();
      if (interrupt_count > 200) { return 1; }
    }

    if (interrupt_count < 120) { return 2; }

    // Wait header OFF (90 interrupts = 4.5ms)
    interrupt_count = 0;

    while ((IOPort0.getPortInputValue() & 0x1) == 1)
    {
      Watchdog.kick();
      if (interrupt_count > 100) { return 3; }
    }

    if (interrupt_count < 80) { return 4; }

    data = 0;

    for (bit_count = 0; bit_count < 32; bit_count++)
    {
      // Wait data bit ON (11 interrupts = 0.562ms)
      interrupt_count = 0;

      while ((IOPort0.getPortInputValue() & 0x1) == 0)
      {
        Watchdog.kick();
        if (interrupt_count > 15) { return 5; }
      }

      if (interrupt_count < 7) { return 6; }

      // Wait data bit OFF (11 interrupts = 0.562ms / 33 interrupts = 1.687ms))
      interrupt_count = 0;

      while ((IOPort0.getPortInputValue() & 0x1) == 1)
      {
        Watchdog.kick();
        if (interrupt_count > 40) { return 7; }
      }

      if (interrupt_count < 7) { return 8; }

      data = data << 1;
      if (interrupt_count > 17) { data |= 1; } 

      if (bit_count == 15)
      {
        //if (data != 0x10ef) { return 9; }
        data = 0;
      }
    }

    return data;
  }

  public void timerInterrupt()
  {
    interrupt_count++;
    servo_count++;

    // After 20ms, reset servo pulse (turn off since transistors reverse
    // logic.  Set servo pulses to the values for the next 20ms cycle.
    if (servo_count == 400)
    {
      IOPort0.setPinsLow(0x06);
      servo_count = 0;
      servo0_curr_value = servo0_new_value;
      servo1_curr_value = servo1_new_value;
    }

    // Set servo pulse to 0 (reverse logic because of transistor).
    if (servo0_curr_value == servo_count) { IOPort0.setPinsHigh(0x02); }
    if (servo1_curr_value == servo_count) { IOPort0.setPinsHigh(0x04); }
  }
}


